@@ -46,7 +46,7 @@ internationalized, to the local language and cultural habits.
 #   find this format documented anywhere.
 
 
-import locale, copy, io, os, re, struct, sys
+import locale, copy, os, re, struct, sys
 from errno import ENOENT
 
 
@@ -57,7 +57,21 @@ __all__ = ['NullTranslations', 'GNUTranslations', 'Catalog',
            'ldngettext', 'lngettext', 'ngettext',
            ]
 
-_default_localedir = os.path.join(sys.base_prefix, 'share', 'locale')
+_default_localedir = os.path.join(sys.prefix, 'share', 'locale')
+
+
+def test(condition, true, false):
+    """
+    Implements the C expression:
+
+      condition ? true : false
+
+    Required to correctly interpret plural forms.
+    """
+    if condition:
+        return true
+    else:
+        return false
 
 
 def c2py(plural):
@@ -65,15 +79,20 @@ def c2py(plural):
     Python lambda function that implements an equivalent expression.
     """
     # Security check, allow only the "n" identifier
+    try:
+        from cStringIO import StringIO
+    except ImportError:
+        from StringIO import StringIO
     import token, tokenize
-    tokens = tokenize.generate_tokens(io.StringIO(plural).readline)
+    tokens = tokenize.generate_tokens(StringIO(plural).readline)
     try:
         danger = [x for x in tokens if x[0] == token.NAME and x[1] != 'n']
     except tokenize.TokenError:
-        raise ValueError('plural forms expression error, maybe unbalanced parenthesis')
+        raise ValueError, \
+              'plural forms expression error, maybe unbalanced parenthesis'
     else:
         if danger:
-            raise ValueError('plural forms expression could be dangerous')
+            raise ValueError, 'plural forms expression could be dangerous'
 
     # Replace some C operators by their Python equivalents
     plural = plural.replace('&&', ' and ')
@@ -83,11 +102,11 @@ def c2py(plural):
     plural = expr.sub(' not \\1', plural)
 
     # Regular expression and replacement function used to transform
-    # "a?b:c" to "b if a else c".
+    # "a?b:c" to "test(a,b,c)".
     expr = re.compile(r'(.*?)\?(.*?):(.*)')
     def repl(x):
-        return "(%s if %s else %s)" % (x.group(2), x.group(1),
-                                       expr.sub(repl, x.group(3)))
+        return "test(%s, %s, %s)" % (x.group(1), x.group(2),
+                                     expr.sub(repl, x.group(3)))
 
     # Code to transform the plural expression, taking care of parentheses
     stack = ['']
@@ -99,7 +118,7 @@ def c2py(plural):
                 # Actually, we never reach this code, because unbalanced
                 # parentheses get caught in the security check at the
                 # beginning.
-                raise ValueError('unbalanced parenthesis in plural form')
+                raise ValueError, 'unbalanced parenthesis in plural form'
             s = expr.sub(repl, stack.pop())
             stack[-1] += '(%s)' % s
         else:
@@ -110,35 +129,36 @@ def c2py(plural):
 
 
 
-def _expand_lang(loc):
-    loc = locale.normalize(loc)
+def _expand_lang(locale):
+    from locale import normalize
+    locale = normalize(locale)
     COMPONENT_CODESET   = 1 << 0
     COMPONENT_TERRITORY = 1 << 1
     COMPONENT_MODIFIER  = 1 << 2
     # split up the locale into its base components
     mask = 0
-    pos = loc.find('@')
+    pos = locale.find('@')
     if pos >= 0:
-        modifier = loc[pos:]
-        loc = loc[:pos]
+        modifier = locale[pos:]
+        locale = locale[:pos]
         mask |= COMPONENT_MODIFIER
     else:
         modifier = ''
-    pos = loc.find('.')
+    pos = locale.find('.')
     if pos >= 0:
-        codeset = loc[pos:]
-        loc = loc[:pos]
+        codeset = locale[pos:]
+        locale = locale[:pos]
         mask |= COMPONENT_CODESET
     else:
         codeset = ''
-    pos = loc.find('_')
+    pos = locale.find('_')
     if pos >= 0:
-        territory = loc[pos:]
-        loc = loc[:pos]
+        territory = locale[pos:]
+        locale = locale[:pos]
         mask |= COMPONENT_TERRITORY
     else:
         territory = ''
-    language = loc
+    language = locale
     ret = []
     for i in range(mask+1):
         if not (i & ~mask):  # if all components for this combo exist ...
@@ -196,6 +216,19 @@ class NullTranslations:
         else:
             return msgid2
 
+    def ugettext(self, message):
+        if self._fallback:
+            return self._fallback.ugettext(message)
+        return unicode(message)
+
+    def ungettext(self, msgid1, msgid2, n):
+        if self._fallback:
+            return self._fallback.ungettext(msgid1, msgid2, n)
+        if n == 1:
+            return unicode(msgid1)
+        else:
+            return unicode(msgid2)
+
     def info(self):
         return self._info
 
@@ -208,24 +241,25 @@ class NullTranslations:
     def set_output_charset(self, charset):
         self._output_charset = charset
 
-    def install(self, names=None):
-        import builtins
-        builtins.__dict__['_'] = self.gettext
+    def install(self, unicode=False, names=None):
+        import __builtin__
+        __builtin__.__dict__['_'] = unicode and self.ugettext or self.gettext
         if hasattr(names, "__contains__"):
             if "gettext" in names:
-                builtins.__dict__['gettext'] = builtins.__dict__['_']
+                __builtin__.__dict__['gettext'] = __builtin__.__dict__['_']
             if "ngettext" in names:
-                builtins.__dict__['ngettext'] = self.ngettext
+                __builtin__.__dict__['ngettext'] = (unicode and self.ungettext
+                                                             or self.ngettext)
             if "lgettext" in names:
-                builtins.__dict__['lgettext'] = self.lgettext
+                __builtin__.__dict__['lgettext'] = self.lgettext
             if "lngettext" in names:
-                builtins.__dict__['lngettext'] = self.lngettext
+                __builtin__.__dict__['lngettext'] = self.lngettext
 
 
 class GNUTranslations(NullTranslations):
     # Magic number of .mo files
-    LE_MAGIC = 0x950412de
-    BE_MAGIC = 0xde120495
+    LE_MAGIC = 0x950412deL
+    BE_MAGIC = 0xde120495L
 
     def _parse(self, fp):
         """Override this method to support alternative .mo formats."""
@@ -246,10 +280,10 @@ class GNUTranslations(NullTranslations):
             version, msgcount, masteridx, transidx = unpack('>4I', buf[4:20])
             ii = '>II'
         else:
-            raise OSError(0, 'Bad magic number', filename)
+            raise IOError(0, 'Bad magic number', filename)
         # Now put all messages from the .mo file buffer into the catalog
         # dictionary.
-        for i in range(0, msgcount):
+        for i in xrange(0, msgcount):
             mlen, moff = unpack(ii, buf[masteridx:masteridx+8])
             mend = moff + mlen
             tlen, toff = unpack(ii, buf[transidx:transidx+8])
@@ -258,13 +292,13 @@ class GNUTranslations(NullTranslations):
                 msg = buf[moff:mend]
                 tmsg = buf[toff:tend]
             else:
-                raise OSError(0, 'File is corrupt', filename)
+                raise IOError(0, 'File is corrupt', filename)
             # See if we're looking at GNU .mo conventions for metadata
             if mlen == 0:
                 # Catalog description
                 lastk = None
-                for b_item in tmsg.split('\n'.encode("ascii")):
-                    item = b_item.decode().strip()
+                for item in tmsg.splitlines():
+                    item = item.strip()
                     if not item:
                         continue
                     k = v = None
@@ -291,20 +325,38 @@ class GNUTranslations(NullTranslations):
             # cause no problems since us-ascii should always be a subset of
             # the charset encoding.  We may want to fall back to 8-bit msgids
             # if the Unicode conversion fails.
-            charset = self._charset or 'ascii'
-            if b'\x00' in msg:
+            if '\x00' in msg:
                 # Plural forms
-                msgid1, msgid2 = msg.split(b'\x00')
-                tmsg = tmsg.split(b'\x00')
-                msgid1 = str(msgid1, charset)
-                for i, x in enumerate(tmsg):
-                    catalog[(msgid1, i)] = str(x, charset)
+                msgid1, msgid2 = msg.split('\x00')
+                tmsg = tmsg.split('\x00')
+                if self._charset:
+                    msgid1 = unicode(msgid1, self._charset)
+                    tmsg = [unicode(x, self._charset) for x in tmsg]
+                for i in range(len(tmsg)):
+                    catalog[(msgid1, i)] = tmsg[i]
             else:
-                catalog[str(msg, charset)] = str(tmsg, charset)
+                if self._charset:
+                    msg = unicode(msg, self._charset)
+                    tmsg = unicode(tmsg, self._charset)
+                catalog[msg] = tmsg
             # advance to next entry in the seek tables
             masteridx += 8
             transidx += 8
 
+    def gettext(self, message):
+        missing = object()
+        tmsg = self._catalog.get(message, missing)
+        if tmsg is missing:
+            if self._fallback:
+                return self._fallback.gettext(message)
+            return message
+        # Encode the Unicode tmsg back to an 8-bit string, if possible
+        if self._output_charset:
+            return tmsg.encode(self._output_charset)
+        elif self._charset:
+            return tmsg.encode(self._charset)
+        return tmsg
+
     def lgettext(self, message):
         missing = object()
         tmsg = self._catalog.get(message, missing)
@@ -316,6 +368,22 @@ class GNUTranslations(NullTranslations):
             return tmsg.encode(self._output_charset)
         return tmsg.encode(locale.getpreferredencoding())
 
+    def ngettext(self, msgid1, msgid2, n):
+        try:
+            tmsg = self._catalog[(msgid1, self.plural(n))]
+            if self._output_charset:
+                return tmsg.encode(self._output_charset)
+            elif self._charset:
+                return tmsg.encode(self._charset)
+            return tmsg
+        except KeyError:
+            if self._fallback:
+                return self._fallback.ngettext(msgid1, msgid2, n)
+            if n == 1:
+                return msgid1
+            else:
+                return msgid2
+
     def lngettext(self, msgid1, msgid2, n):
         try:
             tmsg = self._catalog[(msgid1, self.plural(n))]
@@ -330,30 +398,30 @@ class GNUTranslations(NullTranslations):
             else:
                 return msgid2
 
-    def gettext(self, message):
+    def ugettext(self, message):
         missing = object()
         tmsg = self._catalog.get(message, missing)
         if tmsg is missing:
             if self._fallback:
-                return self._fallback.gettext(message)
-            return message
+                return self._fallback.ugettext(message)
+            return unicode(message)
         return tmsg
 
-    def ngettext(self, msgid1, msgid2, n):
+    def ungettext(self, msgid1, msgid2, n):
         try:
             tmsg = self._catalog[(msgid1, self.plural(n))]
         except KeyError:
             if self._fallback:
-                return self._fallback.ngettext(msgid1, msgid2, n)
+                return self._fallback.ungettext(msgid1, msgid2, n)
             if n == 1:
-                tmsg = msgid1
+                tmsg = unicode(msgid1)
             else:
-                tmsg = msgid2
+                tmsg = unicode(msgid2)
         return tmsg
 
 
 # Locate a .mo file using the gettext strategy
-def find(domain, localedir=None, languages=None, all=False):
+def find(domain, localedir=None, languages=None, all=0):
     # Get some reasonable defaults for arguments that were not supplied
     if localedir is None:
         localedir = _default_localedir
@@ -412,11 +480,11 @@ def translation(domain, localedir=None, languages=None,
                 class_=None, fallback=False, codeset=None):
     if class_ is None:
         class_ = GNUTranslations
-    mofiles = find(domain, localedir, languages, all=True)
+    mofiles = find(domain, localedir, languages, all=1)
     if not mofiles:
         if fallback:
             return NullTranslations()
-        raise OSError(ENOENT, 'No translation file found for domain', domain)
+        raise IOError(ENOENT, 'No translation file found for domain', domain)
     # Avoid opening, reading, and parsing the .mo file after it's been done
     # once.
     result = None
@@ -439,9 +507,9 @@ def translation(domain, localedir=None, languages=None,
     return result
 
 
-def install(domain, localedir=None, codeset=None, names=None):
+def install(domain, localedir=None, unicode=False, codeset=None, names=None):
     t = translation(domain, localedir, fallback=True, codeset=codeset)
-    t.install(names)
+    t.install(unicode, names)
 
 
 
@@ -478,7 +546,7 @@ def dgettext(domain, message):
     try:
         t = translation(domain, _localedirs.get(domain, None),
                         codeset=_localecodesets.get(domain))
-    except OSError:
+    except IOError:
         return message
     return t.gettext(message)
 
@@ -486,7 +554,7 @@ def ldgettext(domain, message):
     try:
         t = translation(domain, _localedirs.get(domain, None),
                         codeset=_localecodesets.get(domain))
-    except OSError:
+    except IOError:
         return message
     return t.lgettext(message)
 
@@ -494,7 +562,7 @@ def dngettext(domain, msgid1, msgid2, n):
     try:
         t = translation(domain, _localedirs.get(domain, None),
                         codeset=_localecodesets.get(domain))
-    except OSError:
+    except IOError:
         if n == 1:
             return msgid1
         else:
@@ -505,7 +573,7 @@ def ldngettext(domain, msgid1, msgid2, n):
     try:
         t = translation(domain, _localedirs.get(domain, None),
                         codeset=_localecodesets.get(domain))
-    except OSError:
+    except IOError:
         if n == 1:
             return msgid1
         else:
